@@ -115,7 +115,7 @@ class ContinuousTimeGaussianDiffusion(nn.Module):
         loss_type = 'l1',
         noise_schedule = 'linear',
         num_sample_steps = 500,
-        clip_after_noising_during_sampling = False,
+        clip_sample_denoised = True,
         learned_schedule_net_hidden_dim = 1024,
         learned_noise_schedule_frac_gradient = 1.   # between 0 and 1, determines what percentage of gradients go back, so one can update the learned noise schedule more slowly
     ):
@@ -150,10 +150,7 @@ class ContinuousTimeGaussianDiffusion(nn.Module):
         # sampling
 
         self.num_sample_steps = num_sample_steps
-
-        # clipping related hyperparameters
-
-        self.clip_after_noising_during_sampling = clip_after_noising_during_sampling
+        self.clip_sample_denoised = clip_sample_denoised
 
     @property
     def device(self):
@@ -172,9 +169,6 @@ class ContinuousTimeGaussianDiffusion(nn.Module):
         # reviewer found an error in the equation in the paper (missing sigma)
         # following - https://openreview.net/forum?id=2LdBqxc1Yv&noteId=rIQgH0zKsRt
 
-        # todo - derive x_start from the posterior mean and do dynamic thresholding
-        # assumed that is what is going on in Imagen
-
         log_snr = self.log_snr(time)
         log_snr_next = self.log_snr(time_next)
         c = -expm1(log_snr - log_snr_next)
@@ -182,10 +176,21 @@ class ContinuousTimeGaussianDiffusion(nn.Module):
         squared_alpha, squared_alpha_next = log_snr.sigmoid(), log_snr_next.sigmoid()
         squared_sigma, squared_sigma_next = (-log_snr).sigmoid(), (-log_snr_next).sigmoid()
 
+        alpha, sigma, alpha_next = map(sqrt, (squared_alpha, squared_sigma, squared_alpha_next))
+
         batch_log_snr = repeat(log_snr, ' -> b', b = x.shape[0])
         pred_noise = self.denoise_fn(x, batch_log_snr)
 
-        model_mean = sqrt(squared_alpha_next / squared_alpha) * (x - c * sqrt(squared_sigma) * pred_noise)
+        if self.clip_sample_denoised:
+            x_start = (x - sigma * pred_noise) / alpha
+
+            # in Imagen, this was changed to dynamic thresholding
+            x_start.clamp_(-1., 1.)
+
+            model_mean = alpha_next / alpha * x * (1 - c) + alpha_next * c * x_start
+        else:
+            model_mean = alpha_next / alpha * (x - c * sigma * pred_noise)
+
         posterior_variance = squared_sigma_next * c
 
         return model_mean, posterior_variance
@@ -216,12 +221,9 @@ class ContinuousTimeGaussianDiffusion(nn.Module):
             times_next = steps[i + 1]
             img = self.p_sample(img, times, times_next)
 
-            if self.clip_after_noising_during_sampling:
-                # clip after noise is added. perhaps this is sufficient for Imagen dynamic thresholding?
-                img.clamp_(-1., 1.)
-
+        img.clamp_(-1., 1.)
         img = unnormalize_to_zero_to_one(img)
-        return img.clamp(0., 1.)
+        return img
 
     @torch.no_grad()
     def sample(self, batch_size = 16):
